@@ -278,6 +278,19 @@ class Shell():
             elif keyin == curses.KEY_F1:
                 curses.endwin()
                 sys.exit()
+            elif keyin in [9]:
+                choices = self._tabcomplete(buff)
+                if len(choices) == 1:
+                    if len(buff.split()) == 1 and not buff.endswith(' '):
+                        buff = choices[0]
+                    else:
+                        buff = ' '.join(buff.split()[:-1])
+                        buff += ' ' + choices[0]
+                elif len(choices) > 1:
+                    self.put("    ".join(choices))
+                elif len(choices) == 0:
+                    pass
+                self._redraw_buffer(buff)
             elif keyin in [260, 261]:
                 if keyin == 260:
                     newx = max(_x - 1, len(self.prompt))
@@ -292,6 +305,35 @@ class Shell():
         self.stdscr.refresh()
         return buff
 
+    def _tabcomplete(self, buff):
+        menu = self.get_menu()
+        commands = []
+        if menu:
+            commands = menu.commands
+        output = []
+        if len(buff.split()) <= 1 and ' ' not in buff:
+            for command in commands:
+                if command.name.startswith(buff):
+                    output.append(command.name)
+                for alias in command.aliases:
+                    if alias.startswith(buff):
+                        output.append(alias)
+        else:
+            command = self._get_command(buff)
+            if command:
+                output = command._tabcomplete(buff)
+        return output
+
+    def _get_command(self, buff):
+        menu = self.get_menu()
+        commands = []
+        if menu:
+            commands = menu.commands
+        for command in commands:
+            if command.name == buff.split()[0]:
+                return command
+        return None
+
     def _redraw_buffer(self, buff):
         """
         Clear the bottom line and re-print the given string on that line
@@ -373,7 +415,11 @@ class Shell():
                 commands = menu.commands
             for command in commands:
                 if tokens[0] == command.name or tokens[0] in command.aliases:
-                    args, kwargs = command.parse_command(tokens)
+                    try:
+                        args, kwargs = command.parse_command(tokens)
+                    except Exception as e:
+                        self.put(e)
+                        break
                     success, message = command.validate(*args, **kwargs)
                     if not success:
                         self.put(message)
